package com.sun.xml.internal.ws.api.server;

import javax.xml.ws.Provider;
import java.lang.reflect.Method;
import com.sun.xml.internal.ws.server.ServerRtException;
import com.sun.xml.internal.ws.resources.WsservletMessages;
import java.util.logging.Level;
import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.resources.ServerMessages;
import com.sun.xml.internal.ws.server.SingletonResolver;
import javax.xml.ws.WebServiceContext;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.message.Packet;
import java.util.logging.Logger;

public abstract class InstanceResolver<T>
{
    private static final Logger logger;
    
    public InstanceResolver() {
    }
    
    @NotNull
    public abstract T resolve(@NotNull Packet p0);
    
    public void postInvoke(@NotNull Packet request, @NotNull T servant) {
    }
    
    public void start(@NotNull WSWebServiceContext wsc, @NotNull WSEndpoint endpoint) {
        this.start(wsc);
    }
    
    @Deprecated
    public void start(@NotNull WebServiceContext wsc) {
    }
    
    public void dispose() {
    }
    
    public static <T> InstanceResolver<T> createSingleton(T singleton) {
        assert singleton != null;
        InstanceResolver<Object> ir = createFromInstanceResolverAnnotation(singleton.getClass());
        if (ir == null) {
            ir = new SingletonResolver(singleton);
            return ir;
        }
        return ir;
    }
    
    @Deprecated
    public static <T> InstanceResolver<T> createDefault(@NotNull Class<T> clazz, boolean bool) {
        return createDefault(clazz);
    }
    
    public static <T> InstanceResolver<T> createDefault(@NotNull Class<T> clazz) {
        InstanceResolver<T> ir = createFromInstanceResolverAnnotation(clazz);
        if (ir == null) {
            ir = new SingletonResolver(createNewInstance(clazz));
            return ir;
        }
        return ir;
    }
    
    public static <T> InstanceResolver<T> createFromInstanceResolverAnnotation(@NotNull Class<T> clazz) {
        Annotation[] annotations = clazz.getAnnotations();
        int length = annotations.length;
        int i = 0;
        while (i < length) {
            Annotation a = annotations[i];
            InstanceResolverAnnotation ira = (InstanceResolverAnnotation)a.annotationType().getAnnotation(InstanceResolverAnnotation.class);
            if (ira == null) {
                ++i;
            }
            else {
                Class<? extends InstanceResolver> ir = ira.value();
                try {
                    return (InstanceResolver)ir.getConstructor(new Class[] { Class.class }).newInstance(new Object[] { clazz });
                }
                catch (InstantiationException e) {
                    throw new WebServiceException(ServerMessages.FAILED_TO_INSTANTIATE_INSTANCE_RESOLVER(ir.getName(), a.annotationType(), clazz.getName()));
                }
                catch (IllegalAccessException e2) {
                    throw new WebServiceException(ServerMessages.FAILED_TO_INSTANTIATE_INSTANCE_RESOLVER(ir.getName(), a.annotationType(), clazz.getName()));
                }
                catch (InvocationTargetException e3) {
                    throw new WebServiceException(ServerMessages.FAILED_TO_INSTANTIATE_INSTANCE_RESOLVER(ir.getName(), a.annotationType(), clazz.getName()));
                }
                catch (NoSuchMethodException e4) {
                    throw new WebServiceException(ServerMessages.FAILED_TO_INSTANTIATE_INSTANCE_RESOLVER(ir.getName(), a.annotationType(), clazz.getName()));
                }
            }
        }
        return null;
    }
    
    protected static <T> T createNewInstance(Class<T> cl) {
        try {
            return cl.newInstance();
        }
        catch (InstantiationException e) {
            InstanceResolver.logger.log(Level.SEVERE, e.getMessage(), e);
            throw new ServerRtException(WsservletMessages.ERROR_IMPLEMENTOR_FACTORY_NEW_INSTANCE_FAILED(cl), new Object[0]);
        }
        catch (IllegalAccessException e2) {
            InstanceResolver.logger.log(Level.SEVERE, e2.getMessage(), e2);
            throw new ServerRtException(WsservletMessages.ERROR_IMPLEMENTOR_FACTORY_NEW_INSTANCE_FAILED(cl), new Object[0]);
        }
    }
    
    @NotNull
    public Invoker createInvoker() {
        return new Invoker() {
            @Override
            public void start(@NotNull WSWebServiceContext wsc, @NotNull WSEndpoint endpoint) {
                InstanceResolver.this.start(wsc, endpoint);
            }
            
            @Override
            public void dispose() {
                InstanceResolver.this.dispose();
            }
            
            public Object invoke(Packet p, Method m, Object... args) throws InvocationTargetException, IllegalAccessException {
                T t = InstanceResolver.this.resolve(p);
                try {
                    return MethodUtil.invoke(t, m, args);
                }
                finally {
                    InstanceResolver.this.postInvoke(p, t);
                }
            }
            
            @Override
            public <U> U invokeProvider(@NotNull Packet p, U arg) {
                T t = InstanceResolver.this.resolve(p);
                try {
                    return ((Provider)t).invoke(arg);
                }
                finally {
                    InstanceResolver.this.postInvoke(p, t);
                }
            }
            
            public String toString() {
                return "Default Invoker over " + InstanceResolver.this.toString();
            }
        };
    }
    
    static {
        logger = Logger.getLogger("com.sun.xml.internal.ws.server");
    }
}
